@@ -25,9 +25,9 @@ import os
 from shutil import copyfile
 
 # Third-party imports
-from PyQt5.QtCore import pyqtSlot, Qt, QCoreApplication, QSettings
+from PyQt5.QtCore import pyqtSlot, Qt, QCoreApplication, QSettings,  QFileInfo
 from PyQt5 import QtCore, QtWidgets, QtGui
-from PyQt5.QtWidgets import QProgressBar
+from PyQt5.QtWidgets import QProgressBar, QFileDialog,  QDialog
 
 # Local application imports
 from pycirkuit.ui.Ui_mainwindow import Ui_MainWindow
@@ -71,6 +71,11 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
         # The app icon is located inside the resource file
         icon = QtGui.QIcon(":/icons/AppIcon")
         self.setWindowIcon(icon)
+        
+        # A class variable to hold the exporting dir temporarily
+        # Set destination dir to the same value of source dir. Defaults to empty.
+        settings = QSettings()
+        self.lastDstDir = settings.value("General/lastSrcDir", "")
 
         # Connect signals with slots
         #NOTE: Is NOT necessary to MANUALLY connect most signals to slots, as 
@@ -108,7 +113,32 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
         # We're quitting constructor
         self.insideConstructor = False
 
-    def _ask_export_as(self, src, dst):
+
+    def _check_export_dir(self, dir):
+        fdlg = QtWidgets.QFileDialog(self)
+        fdlg.setWindowTitle(_translate("MainWindow", "Enter a writable directory to save into",  "Window Title"))
+        fdlg.setDirectory(dir)
+        fdlg.setFileMode(QtWidgets.QFileDialog.Directory)
+        fdlg.setOptions(QtWidgets.QFileDialog.DontUseNativeDialog | QtWidgets.QFileDialog.ShowDirsOnly)
+        fdlg.setViewMode(QtWidgets.QFileDialog.List)
+        while True:
+            if fdlg.exec() == QtWidgets.QFileDialog.Accepted:
+                newDir = fdlg.selectedFiles()[0]
+                #TODO: Check if selected dir is writable
+                try:
+                    name = os.path.join(newDir, 'test.txt')
+                    with open(name, 'w'):
+                        pass
+                except:
+                    pass
+                return newDir
+            else:
+                #Dialog cancelled,  we must return with the dir name unchanged
+                break
+        return dir
+
+
+    def _ask_export_file(self, dst):
         fdlg = QtWidgets.QFileDialog(self)
         fdlg.setWindowTitle(_translate("MainWindow", "Enter a file to save into",  "Window Title"))
         fdlg.setDirectory(dst)
@@ -116,19 +146,26 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
         fdlg.setOptions(QtWidgets.QFileDialog.DontUseNativeDialog)
         fdlg.setViewMode(QtWidgets.QFileDialog.Detail)
         fdlg.setFilter(QtCore.QDir.AllDirs | QtCore.QDir.Files)
-        fdlg.setNameFilters([
-            _translate("MainWindow", "TikZ files (*.tikz)", "File filter"),
-            _translate("MainWindow", "PDF files (*.pdf)", "File filter"),
-            _translate("MainWindow", "PNG files (*.png)", "File filter"),
-            _translate("MainWindow", "JPEG files (*.jpeg *.jpg)", "File filter"),
-            _translate("MainWindow", "TeX files (*.tex)", "File filter"),
-            _translate("MainWindow", "Any files (*)", "File filter")])
+        fdlg.selectFile(dst)
+        #FIXME: Bad programming style: new format additions are been managed in too many differents places
+        filterList = []
+        n, e = os.path.splitext(dst)
+        if e == '.tikz':
+            filterList.append(_translate("MainWindow", "TikZ files (*.tikz)", "File filter"))
+        elif e == '.pdf':
+            filterList.append(_translate("MainWindow", "PDF files (*.pdf)", "File filter"))
+        elif e == '.png':
+            filterList.append(_translate("MainWindow", "PNG files (*.png)", "File filter"))
+        elif e == '.jpeg':
+            filterList.append(_translate("MainWindow", "JPEG files (*.jpeg *.jpg)", "File filter"))
+        elif e == '.svg':
+            filterList.append(_translate("MainWindow", "SVG files (*.svg)", "File filter"))
+        filterList.append(_translate("MainWindow", "Any files (*)", "File filter"))
+        fdlg.setNameFilters(filterList)
         if fdlg.exec():
             dst = fdlg.selectedFiles()[0]
-            copyfile(src, dst)
-            return True
         fdlg.close()
-        return False
+        return dst
 
 
     def _ask_saving(self):
@@ -145,6 +182,21 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
             self.actionSave.trigger()
 
 
+    def _ask_writable_dir(self, offendingDir):
+        fdlg = QFileDialog(self)
+        fdlg.setWindowTitle(_translate("MainWindow", "Enter new directory", "File Dialog Title"))
+        fdlg.setDirectory(offendingDir)
+        fdlg.setFileMode(QFileDialog.Directory)
+        fdlg.setOptions(QFileDialog.ShowDirsOnly | QFileDialog.DontUseNativeDialog)
+        fdlg.setViewMode(QFileDialog.Detail)
+        fdlg.setFilter(QtCore.QDir.Dirs | QtCore.QDir.Hidden)
+        if fdlg.exec():
+            newPath = fdlg.selectedFiles()
+        else:
+           newPath = None
+        fdlg.close()
+        return newPath
+
     def _center(self):
         # Get desktop's geometry
         app = QtWidgets.QApplication.instance()
@@ -206,6 +258,26 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
             return False
 
 
+
+    def _ensure_writable_location(self, dstDir):
+        while True:
+            dstDirInfo = QFileInfo(dstDir)
+            if ( dstDirInfo.isDir() and dstDirInfo.isWritable() ):
+                return dstDir
+            else:
+                # Display error messagebox
+                msgBox = QtWidgets.QMessageBox(self)
+                msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Error",  "Message Box title"))
+                msgBox.setIcon(QtWidgets.QMessageBox.Critical)
+                msgBox.setText(_translate("MessageBox", "The exporting destination directory is not writable.", "Message box text." ))
+                msgBox.setInformativeText(_translate("MessageBox", "Please enter a suitable directory to write into.",  "Message Box text"))
+                msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
+                msgBox.exec()
+                # Ask user to choose another directory
+                dstDir = self._ask_writable_dir(dstDirInfo.canonicalPath())[0]
+
+
+
     @pyqtSlot(PyCirkuitError)
     def _display_error(self, error):
         # Open MessageBox and inform user
@@ -262,11 +334,14 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
         # Check that file is not a broken link
         fileName = os.path.normpath(fileName)
         if os.path.isfile(fileName):
-            lastWD, filename = os.path.split(fileName)
+            lastSrcDir, filename = os.path.split(fileName)
             # Change system working dir to target's dir
-            os.chdir(lastWD)
-            settings.setValue("General/lastWD", lastWD)
+            os.chdir(lastSrcDir)
+            # Make new source dir persistent
+            settings.setValue("General/lastSrcDir", lastSrcDir)
             settings.sync()
+            # Destination dir for saving and exporting will be equal to source dir initially
+            self.lastDstDir = lastSrcDir
             with open(filename, 'r') as f:
                 txt = f.read()
                 self.sourceText.setPlainText(txt)
@@ -299,25 +374,30 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
             self.resizeDocks([self.previewWidget], [rect.width() + extraWidth +2], Qt.Horizontal)
 
 
+
     def _save_buffer(self,  dst):
+        """ Saves the content of the editing buffer onto disk.
+        
+        Arguments:
+        dst -- the destination directory
+        """
+        # We decided to handle exceptions
         try:
-            f = open(dst,'w', encoding='UTF-8')
-            f.write(self.sourceText.toPlainText())
-        except OSError as e:
-            errMsg = _translate("MessageBox", "Error saving source file: ", "Error message") + e.strerror + ".\n\n"
-            errMsg += _translate("MessageBox", "Cannot execute command.", "Error message")
+            with open(dst,'w', encoding='UTF-8') as f:
+                f.write(self.sourceText.toPlainText())
+        except FileNotFoundError:
+            raise
+        except PermissionError:
+            errMsg = _translate("MessageBox", "Error saving source file: Destination is not writable.\n\n", "Error message")
+            errMsg += _translate("MessageBox", "Please select a suitable destination to save into.", "Error message")
             QtWidgets.QMessageBox.critical(self, _translate("MessageBox", "PyCirkuit - Error", "Message Box title"),  errMsg)
-            return
+            raise
         else:
-            settings = QSettings()
-            lastWD, self.openedFilename = os.path.split(dst)
-            os.chdir(lastWD)
-            settings.setValue("General/lastWD", lastWD)
+            # If saving succeeded, write down the export path and file name
+            self.lastDstDir, self.openedFilename = os.path.split(dst)
             self.needSaving = False
             self._modify_title()
             self.actionSave.setEnabled(False)
-        finally:
-            f.close()
 
 
     def closeEvent(self,  event):
@@ -376,10 +456,10 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
             self.needSaving = False
         else:
             settings = QSettings()
-            lastWD, filename = os.path.split(newName)
+            lastSrcDir, filename = os.path.split(newName)
             # Change system working dir to target's dir
-            os.chdir(lastWD)
-            settings.setValue("General/lastWD", lastWD)
+            os.chdir(lastSrcDir)
+            settings.setValue("General/lastSrcDir", lastSrcDir)
             settings.sync()
             self.openedFilename = filename
             self.needSaving = True
@@ -398,7 +478,7 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
         # Show the 'load file' Dialog
         fdlg = QtWidgets.QFileDialog(self)
         fdlg.setWindowTitle(_translate("MainWindow", "Source File Selection", "File Dialog title"))
-        fdlg.setDirectory(settings.value("General/lastWD",  ""))
+        fdlg.setDirectory(settings.value("General/lastSrcDir",  ""))
         fdlg.setNameFilters([
             _translate("MainWindow", "PyCirkuit files (*.ckt)", "File filter text"), 
             _translate("MainWindow", "M4 macro files (*.m4)", "File filter text"),
@@ -431,10 +511,10 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
     @pyqtSlot()
     def on_actionSaveAs_triggered(self):
         settings = QSettings()
-        lastWD = settings.value("General/lastWD", QtCore.QStandardPaths.displayName(QtCore.QStandardPaths.HomeLocation))
+        lastSrcDir = settings.value("General/lastSrcDir", QtCore.QStandardPaths.displayName(QtCore.QStandardPaths.HomeLocation))
         fdlg = QtWidgets.QFileDialog(self)
         fdlg.setWindowTitle(_translate("MainWindow", "Enter a file to save into", "File Dialog title"))
-        fdlg.setDirectory(lastWD)
+        fdlg.setDirectory(lastSrcDir)
         fdlg.setFilter(QtCore.QDir.AllDirs | QtCore.QDir.Files)
         fdlg.setNameFilters([
             _translate("MainWindow", "PyCirkuit files (*.ckt)", "File filter"),
@@ -443,21 +523,28 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
         fdlg.setFileMode(QtWidgets.QFileDialog.AnyFile)
         fdlg.setViewMode(QtWidgets.QFileDialog.Detail)
         fdlg.setAcceptMode(QtWidgets.QFileDialog.AcceptSave)
-        if fdlg.exec():
+        while (fdlg.exec() == QDialog.Accepted):
             dst = fdlg.selectedFiles()[0]
-            self._save_buffer(dst)
-        fdlg.close()
+            try:
+                self._save_buffer(dst)
+                return
+            except PermissionError:
+                pass
 
 
     @pyqtSlot()
     def on_actionSave_triggered(self):
-        settings = QSettings()
-        lastWD = settings.value("General/lastWD", "")
-        filePath = os.path.join(lastWD , self.openedFilename)
-        if os.path.isfile(filePath):
-            self._save_buffer(filePath)
-        else:
+        # Try to save into the last destination dir used.
+        filePath = os.path.join(self.lastDstDir , self.openedFilename)
+        # Check empty name
+        if (self.openedFilename == self._translatedUnnamed ):
             self.actionSaveAs.trigger()
+        else:
+            # Try to save. If it fails (non-writable location...), ask for a name.
+            try:
+                self._save_buffer(filePath)
+            except PermissionError:
+                self.actionSaveAs.trigger()
 
 
     @pyqtSlot()
@@ -471,9 +558,9 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
             msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
             response = msgBox.exec()
             return
-        # TODO: The "src" file should be provided by the processor class upon requested
+
+        # Fetch the formats to save. For now the extensions are fixed
         settings = QSettings()
-        lastWD = settings.value("General/lastWD")
         toSave = []
         if settings.value("Export/exportTIKZ", type=bool):
             toSave.append("tikz")
@@ -485,50 +572,64 @@ class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):
             toSave.append("png")
         if settings.value("Export/exportJPEG", type=bool):
             toSave.append("jpeg")
+        
+        # First we must detect if destination dir is writable. If not, ask for another one and update "lastSrcDir" setting (if it's writable)
+        self.lastDstDir = self._ensure_writable_location(self.lastDstDir)
+        
+        # Next we do a loop for every export format requested
         for fileType in toSave:
             src = os.path.join(pycirkuit.__tmpDir__.path(), "cirkuit_tmp") + os.extsep + fileType
-            dst = os.path.join(lastWD, os.path.splitext(self.openedFilename)[0]) + os.extsep + fileType
-            try:    
-                if os.path.exists(dst):
-                    msgBox = QtWidgets.QMessageBox(self)
-                    msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Warning",  "Message Box title"))
-                    msgBox.setIcon(QtWidgets.QMessageBox.Warning)
-                    msgBox.setText(_translate("MessageBox", "There's already a file named \"{filename}\" at working directory.", "Message box text. Don't translate '{filename}'").format(filename=self.openedFilename.partition('.')[0]+os.extsep+fileType))
-                    msgBox.setInformativeText(_translate("MessageBox", "Do you want to overwrite it?",  "Message Box text"))
-                    msgBox.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
-                    saveAsButton = msgBox.addButton(_translate("MessageBox", "Save As...",  "Button text"),  QtWidgets.QMessageBox.AcceptRole)
-                    msgBox.setDefaultButton(QtWidgets.QMessageBox.No)
-                    response = msgBox.exec()
-                    # Overwrite
-                    if response == QtWidgets.QMessageBox.Yes:
+            # First try for destination path
+            dst = os.path.join(self.lastDstDir, os.path.splitext(self.openedFilename)[0]) + os.extsep + fileType
+            success = False
+            while not success:
+                try:    
+                    if os.path.exists(dst):
+                        msgBox = QtWidgets.QMessageBox(self)
+                        msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Warning",  "Message Box title"))
+                        msgBox.setIcon(QtWidgets.QMessageBox.Warning)
+                        msgBox.setText(_translate("MessageBox", "There's already a file named \"{filename}\" at working directory.", "Message box text. Don't translate '{filename}'").format(filename=self.openedFilename.partition('.')[0]+os.extsep+fileType))
+                        msgBox.setInformativeText(_translate("MessageBox", "Do you want to overwrite it?",  "Message Box text"))
+                        msgBox.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
+                        saveAsButton = msgBox.addButton(_translate("MessageBox", "Save As...",  "Button text"),  QtWidgets.QMessageBox.AcceptRole)
+                        msgBox.setDefaultButton(QtWidgets.QMessageBox.No)
+                        response = msgBox.exec()
+                        # Overwrite
+                        if response == QtWidgets.QMessageBox.Yes:
+                            copyfile(src, dst)
+                            success = True
+                        # Save with another name/path (and ask for it first)
+                        if (response == QtWidgets.QMessageBox.NoButton) and (msgBox.clickedButton() == saveAsButton):
+                            dst = self._ask_export_file(dst)
+                            self.lastDstDir, f = os.path.split(dst)
+                            copyfile(src, dst)
+                            success = True
+                        # Any other option means user doesn't want to overwrite the file -> Exit
+                        break
+                    else:
                         copyfile(src, dst)
-                        self.exportButton.setEnabled(False)
-                    # Save with another name (and ask for it first)
-                    if (response == QtWidgets.QMessageBox.NoButton) and (msgBox.clickedButton() == saveAsButton):
-                        if self._ask_export_as(src, dst):
-                            self.exportButton.setEnabled(False)
-                    # Any other option means user doesn't want to overwrite the file -> Exit
-                else:
-                    copyfile(src, dst)
-                    self.exportButton.setEnabled(False)
-            except PermissionError as err:
-                msgBox = QtWidgets.QMessageBox(self)
-                msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Error",  "Message Box title"))
-                msgBox.setIcon(QtWidgets.QMessageBox.Critical)
-                msgBox.setText(_translate("MessageBox", "Permission denied writing the file {filename}.", "Message box text. Don't translate '{filename}'").format(filename=err.filename))
-                msgBox.setInformativeText(_translate("MessageBox", "Please try to export again with another name and/or location.",  "Message Box text"))
-                msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
-                response = msgBox.exec()
-                if self._ask_export_as(src, dst):
-                    self.exportButton.setEnabled(False)
-            except OSError:
-                msgBox = QtWidgets.QMessageBox(self)
-                msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Error",  "Message Box title"))
-                msgBox.setIcon(QtWidgets.QMessageBox.Critical)
-                msgBox.setText(_translate("MessageBox", "An error has occurred trying to export the file. The error says:", "Message Box text"))
-                msgBox.setInformativeText(err.strerror)
-                msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
-                response = msgBox.exec()
+                        success = True
+                except PermissionError as err:
+                    msgBox = QtWidgets.QMessageBox(self)
+                    msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Error",  "Message Box title"))
+                    msgBox.setIcon(QtWidgets.QMessageBox.Critical)
+                    msgBox.setText(_translate("MessageBox", "Permission denied writing the file {filename}.", "Message box text. Don't translate '{filename}'").format(filename=err.filename))
+                    msgBox.setInformativeText(_translate("MessageBox", "Please try to export again with another name and/or location.",  "Message Box text"))
+                    msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
+                    response = msgBox.exec()
+                    # Replace dst with the new user-choosed file
+                    self.lastDstDir = self._ask_export_file(dst)
+                except OSError:
+                    msgBox = QtWidgets.QMessageBox(self)
+                    msgBox.setWindowTitle(_translate("MessageBox", "PyCirkuit - Error",  "Message Box title"))
+                    msgBox.setIcon(QtWidgets.QMessageBox.Critical)
+                    msgBox.setText(_translate("MessageBox", "An error has occurred trying to export the file. The error says:", "Message Box text"))
+                    msgBox.setInformativeText(err.strerror)
+                    msgBox.setStandardButtons(QtWidgets.QMessageBox.Ok)
+                    response = msgBox.exec()
+        # Continue after the 'for' loop. If we are here we've saved successfully all the requested formats, so we can disable the button
+        self.exportButton.setEnabled(False)
+
 
 
     @pyqtSlot()
